/*==========================================================================*
 *                                                                          *
 *   DEMO.C  by Karl Keyte                                                  *
 *                                                                          *
 *   Demonstrates the use of the TCU library (by same author!!)             *
 *                                                                          *
 *   The supplied TCU library is LARGE memory model.  This code should be   *
 *   compiled and linked in the same memory model or it WILL FAIL!          *
 *                                                                          *
 *==========================================================================*/


#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include "tcu.h"          /* Utilities include file */


static FORM      form;    /* Form object */


void main (void)
{
   int           age_id,        /* To hold field ID of age form field */
                 i,             /* Temporary loop counter */
                 name_id,       /* To hold field ID of name form field */
                 pos_x, pos_y,  /* Position for the form */
                 rkey,          /* Key used to exit form */
                 select;        /* Menu selction */
   char          buf[83];       /* Prompt input buffer */
   MENU          mymenu;        /* Menu object */
   FIELD_VALUE   fval;          /* Form field value object */
   FORM_INFO     f_info;        /* Form information structure */

   void far      help (int);                     /* Form help function */
   int far       test_age (int, FIELD_VALUE *);  /* Age field verification */
   int far       fn_keys (int, int);             /* Function key handler */

   char *option_list[] = { "Add Customer           =>",   /* Menu options */
                           "Delete Customer        =>",
                           "Query Customer         =>",
                           "Print Customer Details",
                           "About this Demo",
                           "Go back to DOS",
                           NULL};

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

/* Define the menu */

   select = define_menu (&mymenu,
                         "Customer Menu",
                         colour_attrib (YELLOW, CYAN),       /* Title */
                         colour_attrib (BLUE, CYAN),         /* Box */
                         colour_attrib (WHITE, MAGENTA),     /* Options */
                         colour_attrib (WHITE, BLUE),        /* Selected */
                         colour_attrib (LIGHTGRAY, MAGENTA), /* Deselected */
                         ESC_ESC | ESC_CNTL_C | ESC_PGUP | ESC_FUNC,
                         BOX_DOUBLE,
                         option_list);

   if (select != MENU_OK) {
      printf ("Error in defining menu!\n");
      exit (1);
   }


   load_form (&form, "sample");              /* Load form from CUF file */

   get_field_id (&form, "name", &name_id);   /* Get name field ID */
   get_field_id (&form, "age", &age_id);     /* Get age field ID */

   get_form_info (&form, &f_info);           /* Get form information */
   pos_x = 81-f_info.width;                  /* Calculate where to put */
   pos_y = 26-f_info.height;                 /* the form. */

   set_form_fnkey_fn (&form, fn_keys);       /* Establish fn. key handler */


   i = 1;
   while (set_field_help (&form, i, help) == FORM_OK)  /* Set all fields to */
      i++;                                             /* use the help fn. */

   set_field_verify (&form, age_id, test_age);         /* Verify age field */


   set_menu_option (&mymenu, 4, 0);          /* Disable menu option 4 */


   select = display_menu (&mymenu, 10, 10);  /* Display the menu */

   buf[2] = '\0';       /* Empty starting string for name prompt */

/* Now get the user's menu selction. Keep getting it until an escape key */
/* is pressed or the 'Go back to DOS' option is selcted (select = 6).    */

   while ((select = read_menu_selection (&mymenu))> 0 && select < 6) {

      if (select == 5) {   /* ...about this demo... */

         prepare_notice (colour_attrib (LIGHTBLUE, LIGHTGRAY),
                         colour_attrib (BLUE, LIGHTGRAY),
                         BOX_DOUBLE);

         notice_text ("-------------- ABOUT THIS DEMO ----------------");
         notice_text ("");
         notice_text ("This demonstration uses some of the functions");
         notice_text ("provided in the TCU package to show the use");
         notice_text ("of menus, prompts, notices and forms.  The file");
         notice_text ("TCU.DOC describes the syntax and functions of");
         notice_text ("each routine provided.");

         display_notice (30, 5);    /* Put the notice up */

         clear_notice ();           /* Get rid of notice definition */

      } else {   /* ...otherwise get ready for user's input */

         prepare_notice (colour_attrib (BLACK, GREEN),
                         colour_attrib (WHITE, GREEN),
                         BOX_SINGLE);

         notice_text ("Customer Name :");   /* Prompt bit for input */

         buf[0] = 30;    /* Maximum length of input string */
         prompt_input (17, 1, buf, colour_attrib (YELLOW, RED)); /* Prompt */

         display_notice (20, 11+select);   /* Put up prompt & get string */

         clear_notice ();                  /* Get rid of prompt definition */

         fval.v_string = buf+2;             /* Load form name field with */
         put_field (&form, name_id, &fval); /* the string just obtained */

         set_field_mode (&form, name_id, FORM_NOEDIT); /* No changing name! */

         display_form (&form, pos_x, pos_y); /* Display the form and allow */
         edit_form (&form, 1, &rkey);        /* interactive edits */

         remove_form (&form);               /* Remove the form from screen */
      }
   }


   unload_form (&form);           /* Unload the form definition from memory */
   remove_menu (&mymenu);         /* Remove the menu from the screen */


/* This next section is just to demonstrate the way escape keys work from */
/* menus.  It shows what key was used to leave the menu */


   if (select == 0)
      printf ("Error in displaying menu!\n");
   else
      if (select < 0) {
         select = -select;
         switch (select) {
            case ESC_ESC:    printf ("User cancelled with ESC key\n");
                             break;
            case ESC_PGUP:   printf ("User escaped with PgUp key\n");
                             break;
            case ESC_PGDN:   printf ("User escaped with PgDn key\n");
                             break;
            case ESC_CLEFT:  printf ("User escaped with <--- key\n");
                             break;
            case ESC_CRIGHT: printf ("User escaped with ---> key\n");
                             break;
            case ESC_CNTL_C: printf ("User escaped with CNTL/C\n");
                             break;
            case ESC_FUNC:   printf ("User escaped with F-%d key\n",
                                     escape_fkey ());
                             break;
            default:         printf ("??? Unknown escape key!\n");
                             break;
         }
      }
}


#pragma warn -par
int far test_age (int field, FIELD_VALUE *val)
{
   if (val->v_int == 50) {

      prepare_notice (colour_attrib (YELLOW, RED),
                      colour_attrib (YELLOW, RED),
                      BOX_SINGLE);

      notice_text ("The AGE must be a number from 18 to");
      notice_text ("120, excluding 50 which is a boring");
      notice_text ("sort of age!");

      display_notice (15, 9);
      clear_notice ();

      return (0);    /* Failed verification */
   }

   return (1);       /* Passed verification */
}
#pragma warn +par


void far help (int field)
{

   prepare_notice (colour_attrib (WHITE, GREEN),
                   colour_attrib (WHITE, GREEN),
                   BOX_DOUBLE);

   notice_text ("Field : %03d", field);
   notice_text ("");
   notice_text ("Please complete the form displayed and");
   notice_text ("finish your entry with:");
   notice_text ("");
   notice_text ("    ESC   : To abort the entry");
   notice_text (" or PgUp  : To complete the entry");
   notice_text ("");
   notice_text ("      PRESS RETURN TO CONTINUE");

   display_notice (19, 6);
   clear_notice ();
}


/* Function key handler */

int far fn_keys (int field, int key)
{
   prepare_notice (colour_attrib (WHITE, GREEN),   /* Reject all function */
                   colour_attrib (WHITE, GREEN),   /* keys. */
                   BOX_DOUBLE);

   notice_text ("You just pressed F-%d on field %d which is undefined!",
                key, field);
   display_notice (15, 12);
   clear_notice ();

   return (0);          /* 0 means continue as if no key was pressed */
}
